/*
 * This file is (c) Copyright Zarafa, The Netherlands, 2007
 *
 * Use of this file and the source contained within it is subject to the
 * license of the package it belongs to. See the LICENSE file in the
 * top-level directory of the package.
 *
 */

#include "platform.h"

// Damn windows header defines max which break C++ header files
#undef max

#include <pthread.h>
#include <mapix.h>
#include <time.h>
#include <iostream>

#include "ECMapiUtils.h"

// Returns the FILETIME as GM-time
FILETIME vmimeDatetimeToFiletime(vmime::datetime dt) {
	FILETIME sFiletime;
	struct tm when;
	int iYear, iMonth, iDay, iHour, iMinute, iSecond, iZone;
	time_t lTmpTime;

	dt.getDate( iYear, iMonth, iDay );	
	dt.getTime( iHour, iMinute, iSecond, iZone );

	when.tm_hour	= iHour;	
	when.tm_min		= iMinute - iZone; // Zone is expressed in minutes. mktime() will normalize negative values or values over 60
	when.tm_sec		= iSecond;
	when.tm_mon		= iMonth - 1;	
	when.tm_mday	= iDay;	
	when.tm_year	= iYear - 1900;
	when.tm_isdst	= -1;		// ignore dst

	lTmpTime = convertGMTime(&when);

	UnixTimeToFileTime(lTmpTime, &sFiletime);
	return sFiletime;
}

vmime::datetime FiletimeTovmimeDatetime(FILETIME ft) {
	time_t tmp;
	struct tm convert;

	FileTimeToUnixTime(ft, &tmp);
	gmtime_safe(&tmp, &convert);
	return vmime::datetime(convert.tm_year + 1900, convert.tm_mon + 1, convert.tm_mday,
					convert.tm_hour, convert.tm_min, convert.tm_sec);
}


#define BUFSIZE 4096
HRESULT readStream(IStream *lpStream, std::string &strData) {
	HRESULT hr = hrSuccess;
	ULONG ulRead = 0;
	char buffer[BUFSIZE];
	LARGE_INTEGER zero = {{0,0}};

	hr = lpStream->Seek(zero, SEEK_SET, NULL);

	if(hr != hrSuccess)
		goto exit;

	while(1) {
		hr = lpStream->Read(buffer, BUFSIZE, &ulRead);

		if(hr != hrSuccess || ulRead == 0)
			break;

		strData.append(buffer, ulRead);
	}

exit:
	return hr;
}
