/*
 * This file is (c) Copyright Zarafa, The Netherlands, 2007
 *
 * Use of this file and the source contained within it is subject to the
 * license of the package it belongs to. See the LICENSE file in the
 * top-level directory of the package.
 *
 */

#include "platform.h"

// Damn windows header defines max which break C++ header files
#undef max

#include <iostream>

#include "ECVMIMEUtils.h"

#include <mapi.h>
#include <mapitags.h>
#include <mapidefs.h>

#include <mapix.h>
//#include <mapiutil.h>
//#include <mapispi.h>

ECVMIMESender::ECVMIMESender(ECLogger *newlpLogger, std::string strSMTPHost) : ECSender(newlpLogger, strSMTPHost) {
	lpSess = NULL;
	lpTransport = NULL;
}

ECVMIMESender::~ECVMIMESender() {
	if (lpSess)
		delete lpSess;
	if (lpTransport)
		delete lpTransport;
}

#ifdef DEBUG
HRESULT ECVMIMESender::printMail(vmime::message *lpVMMessage) {
	HRESULT hr = hrSuccess;
	string mail;

	try {
		mail = lpVMMessage->generate();
		cout << endl << endl << "----------" << endl << endl <<
			mail << endl << endl << "----------" << endl;
	}
	catch (...) {
		cerr << "bah, exception.\n";
		hr = MAPI_E_CALL_FAILED;
	}
	return hr;
}
#endif

// This function does not catch the vmime exception
// it should be handled by the calling party.
HRESULT ECVMIMESender::sendMail(vmime::message *lpVMMessage) {
	HRESULT hr = hrSuccess;

	// recover from prev. error?
	if (lpTransport) {
		delete lpTransport;
		lpTransport = NULL;
	}
	if (lpSess) {
		delete lpSess;
		lpSess = NULL;
	}
	smtpresult = 0;
	error.clear();

	try {
		// Session initialization (global properties)
		lpSess = new vmime::messaging::session();

		// set the server address and port, plus type of service by use of url
		vmime::utility::url url("smtp://"+smtphost);
		lpTransport = lpSess->getTransport(url, NULL);

		// Connect to the server
		lpTransport->connect();
		lpTransport->send(lpVMMessage);
		lpTransport->disconnect();
	}
	catch (vmime::exceptions::command_error& e) {
		lpLogger->Log(EC_LOGLEVEL_ERROR, "SMTP: %s Response: %s", e.what().c_str(), e.response().c_str());
		smtpresult = atoi(e.response().substr(0, e.response().find_first_of(" ")).c_str());
		error = e.response();
		// message should be cancelled, unsendable, test by smtp result code.
		hr = MAPI_W_CANCEL_MESSAGE;
	} 
	catch (vmime::exception& e) {
		// connection_greeting_error, ...?
		lpLogger->Log(EC_LOGLEVEL_ERROR, e.what());
		error = e.what();
		hr = MAPI_E_NETWORK_ERROR;
	}
	catch (std::exception& e) {
		lpLogger->Log(EC_LOGLEVEL_ERROR, "%s",e.what());
		error = e.what();
		hr = MAPI_E_NETWORK_ERROR;
	}
	if (lpTransport) {
		delete lpTransport;
		lpTransport = NULL;
	}
	if (lpSess) {
		delete lpSess;
		lpSess = NULL;
	}

	return hr;
}


// ---
// c style util functions
// ---

// need this for attachments
// CHECK: is this correct? How about spaces? need to make longfi~.jpg kind of name?
// should always be 8.3 format?
void getShortFilename(std::string& strLongIn, std::string& strShortOut) {
	int	iNewNameLength = 8;
	int iDotPos = strLongIn.find_last_of(".");

	if (iDotPos > 0 && iDotPos < iNewNameLength)
		iNewNameLength = iDotPos;
	
	strShortOut.append(strLongIn.substr(0, iNewNameLength));
	// take iDotPos + 4 ( include the dot ) and not strlength to be sure to cutoff [.mpeg and .jpeg]'ish stuff..
	if (iDotPos > 0)
		strShortOut.append(strLongIn.substr(iDotPos, iDotPos+4));
}

// for debugging purposes
void componentToString(vmime::component& comp, std::string& strOut) {
	vmime::utility::outputStreamStringAdapter os(strOut);
	comp.generate(os);
}

// for debugging purposes
void printComponent(vmime::component& comp) {
	std::string out;

	vmime::utility::outputStreamStringAdapter os(out);
	comp.generate(os);
	std::cout << out.c_str() << std::endl;
}
