/*
 * This file is (c) Copyright Zarafa, The Netherlands, 2007
 *
 * Use of this file and the source contained within it is subject to the
 * license of the package it belongs to. See the LICENSE file in the
 * top-level directory of the package.
 *
 */

#ifndef VMIMETOMAPI
#define VMIMETOMAPI

#include <vmime/vmime.hpp>
#include <mapix.h>
#include "mapidefs.h"
#include "ECLogger.h"
#include "options.h"

// We currently always write PR_BODY and PR_SUBJECT, etc as windows-1252
// charset. This is good enough to support western-european e-mail
// We don't set PR_INTERNET_CPID, and windows therefore assumes windows-1252
#define MAPI_CHARSET vmime::charset(vmime::charsets::WINDOWS_1252)

enum BODYLEVEL { BODY_NONE, BODY_PLAIN, BODY_HTML };

class VMIMEToMAPI
{
public:
	VMIMEToMAPI();
	VMIMEToMAPI(LPADRBOOK lpAdrBook, ECLogger *newlogger, delivery_options dopt);
	virtual	~VMIMEToMAPI();

	HRESULT convertVMIMEToMAPI(string input, IMessage *lpMessage);

private:
	ECLogger *lpLogger;
	bool delete_logger;
	delivery_options m_dopt;
	LPADRBOOK m_lpAdrBook;
	BODYLEVEL bodyLevel;

	HRESULT fillMAPIMail(vmime::message* VMMessage, IMessage *lpMessage);
	HRESULT disectBody(vmime::header* VMHeader, vmime::body* VMBody, IMessage* lpMessage, bool onlyBody = false);

	HRESULT handleHeaders(vmime::header* VMHeader, IMessage* lpMessage);
	HRESULT handleRecipients(vmime::header* VMHeader, IMessage* lpMessage);
	HRESULT modifyRecipientList(LPADRLIST lpRecipients, const vmime::addressList* lpVMAddressList, ULONG ulRecipType);
	HRESULT modifyFromAddressBook(LPSPropValue *lppPropVals, ULONG *lpulValues, const char *email, ULONG ulRecipType, LPSPropTagArray lpPropsList);

	HRESULT handleTextpart(vmime::header* VMHeader, vmime::body* VMBody, IMessage* lpMessage);
	HRESULT handleHTMLTextpart(vmime::header* VMHeader, vmime::body* VMBody, IMessage* lpMessage);
	HRESULT handleAttachment(vmime::header* VMHeader, vmime::body* VMBody, IMessage* lpMessage);
	HRESULT handleMessageToMeProps(IMessage *lpMessage, LPADRLIST lpRecipients);
};

#endif
