/*
 * This file is (c) Copyright Zarafa, The Netherlands, 2007
 *
 * Use of this file and the source contained within it is subject to the
 * license of the package it belongs to. See the LICENSE file in the
 * top-level directory of the package.
 *
 */

#include "mapiAttachment.h"

mapiAttachment::mapiAttachment(const vmime::contentHandler &data, const vmime::encoding &enc, const vmime::mediaType &type, const std::string contentid, const vmime::word filename, const vmime::text &desc) :
	defaultAttachment(data, enc, type, desc)
{
	m_filename = filename;
	m_contentid = contentid;

	m_hasCharset = false;
}

void mapiAttachment::addCharset(vmime::charset ch) {
	m_hasCharset = true;
	m_charset = ch;
}

void mapiAttachment::generatePart(vmime::bodyPart& part) const
{
	vmime::defaultAttachment::generatePart(part);

	vmime::contentDispositionField& cdf = part.getHeader()->ContentDisposition();
	vmime::contentTypeField& ctf = part.getHeader()->ContentType();

	cdf.setFilename(m_filename);
	ctf.getParameter("name")->setValue(m_filename);
	if (m_hasCharset)
		ctf.setCharset(vmime::charset(m_charset));
	
	if (m_contentid != "") {
		// Field is created when accessed through ContentId, so don't create if we have no
		// content-id to set
		vmime::messageIdField& cdi = part.getHeader()->ContentId();
		cdi.setValue(vmime::messageId("<" + m_contentid + ">"));
	}
}

